#!/usr/bin/perl
################################################################################
# Wrapper for tmdb.pl tool for bulk movie metadata updates for mythdora        #
#                                                                              #
# 08/15/2008                                                                   #
# 12/17/2009                                                                   #
# 02/09/2010                                                                   #
# Ryan Pisani <mythdora -AT- thepisanis.com>                                   #
################################################################################
#Parts of this were adopted from imdbbatchupdater.pl which in turn uses much   #
#of the original imdb.pl. This is a simpler wrapper to maintain and it utilizes#
#the standard imdb.pl shipped with mythvideo.                                  #
################################################################################
#version: 1.0 : Initial roll-out works with imdb.pl 1.3                        #
#version: 1.1 : Added some further error checking                              #
#version: 1.2 : Added -Manual flag for manual entering of IMDBIDs              #
#version: 1.3 : Added -fileup metadata update for newly added/removed movies   #
#version: 1.4 : Duplicate menu will default to first entry in the returned     #
#               list.                                                          #
#               Added -Exclude flag to mark any items with non-imdb match with #
#               a bogus imdb number to be skipped in later searches.           #
#               Added -help for extended examples in usage                     #
#version: 1.5 : Added -Folder flag to overide videopath for new video search   #
#               Fixed hash sort of new files so that entries with more than 1  #
#               entry in the DB will not be "true" in new check.               #
#               Fixed a bug in the newvideo find where multiples were added    #
#version:1.5.1: Added showlevel fix in -Fileup to fix incorrect default showlvl#
#version: 1.6 : Found bug in -Title lookups where title was updated blank      #
#version: 1.7 : Added fix for DVD filesystem updates so .vob do no get entered #
#               individually.                                                  #
#               Added a check for the poster file as to not retrieve it again  #
#               if the file changed locations.                                 #
#               Added a check to update filename entry only if you move the    #
#               file to another directory instead of a full lookup as if it    #
#               were new.                                                      #
#version: 1.8 : Corrected issue with the recent version of imdb.pl that        #
#               introduced a gap in the TEMPDATA array. I added a little bit   #
#               more of robust data check to try to prevent mishaps if imdb.pl #
#               changes in the future. (thanks xirtama for the heads up).      #
#               Also added more -Folder functionality. You now can do -Folder  #
#               combined with -N or -A to only target updates on sub-directory.#
#version: 1.9 : Corrected an issue with multiple title handling. If a title    #
#               search returns more than one imdb.com entry it now handles that#
#               situation accordingly.                                         #
#               Added optional -Getcovers argument. This will search for new   #
#               posters for all movies that don't currently have a coverfile   #
#               listed.                                                        #
#version:1.9.1: Corrected an issue with NULL titles when no matches returned   #
#               from imdb.com and the -Exclude flag being specified.           #
#version:1.9.2: Modified the -Fileup option for better handling of files with  #
#               [] & () in the name.                                           #
#               Corrected issue with -Fileup that where duplicates were added  #
#               to the database because of (my) faulty logic and assuming that #
#               users would not use the video manager to import new files.     #
#version: 1.10: Added -Update flag that can be used in conjunction with -A and #
#               -Title searches to update metadata for known IMDB numbers. This#
#               is helpful in the recent case where imdb.pl was returning NULL #
#               plot values.                                                   #
#               Added -Exrecheck flag that is used as a primary parameter to   #
#               re-check for IMDB data for those files you may have previously #
#               excluded. This can be used in conjuntion with -Folder.         #
#version: 1.11: Modified -Fileup to honor users with : separated video dirs in #
#               their mythtv videosetup.                                       #
#               Added -Imdbnum flag that overrides imdbnum retreival. This is  #
#               useful for full directory overrides in the case of a series or #
#               or multipart film. Use carefully as this will retain the passed#
#               imdbnum for all entries. So don't use it on your normal dir.   #
#version: 1.12: Modified -Fileup check to follow symbolic links & autofs dirs. #
#               Fixed a bug in title lookup that caused some titles to return  #
#               a 'failure to retrieve' error incorrectly.                     #
#               Added -Default override so that [0] entry in duplicate entries #
#               is automatically selected.                                     #
#version: 1.13: Added support for genre & cast data from imdb output. (thanks  #
#               Jamie A. for patch). Genre & cast can be updated via -Update -A#
#               for your existing library.                                     #
#version: 1.14: Added inetref equals to null check in the $new mysql statement.#
#               Mythweb apparently adds new video files with no value.         #
#version:1.14.1:Corrected issue in sql syntax for folder search.               #
#version:1.14.2:Corrected issue in multiple video folders                      #
#version:1.14.3:Added videometa country patch as submitted by Niall Ryan       #
#version: 1.15 :Added -Newlevel switch so that default showlevel can be set.   #
#               Changed default imdb grabber to tmdb.pl. Enhanced -Getcovers   #
#               switch to better retreive available images.                    #
#version: 1.16 :Modified the script to work with new tmdb.pl grabber present in#
#               0.23. Poster grabbing, etc will be handled a bit differently.  #
#               Default imdb path is now a bit different as well.              #
#version: 1.17 :Fixed to work with tmdb.py for 0.23 release version.           #
################################################################################
use strict;
use DBI;
use Getopt::Long;
use File::Find;
use File::Basename;
use Sys::Hostname;
use LWP::Simple;

#Set Global Path For Execution
$ENV{PATH}="/bin:/usr/sbin:/usr/bin:/sbin";

#Defaults & global declarations here
my $dbhost = '127.0.0.1';
my $db = 'mythconverg';
my $dbuser = 'mythtv';
my $dbpass = 'mythtv';
my @TITLES;           	#Initial query from the imdb.pl script
my $imdbnum;  		#imdbnum set from imdb.pl
my $movie;  		#movie title set from query
my $intid;  		#incremental id for videometadata table
my @FAILEDLOOKUP;	#array to store failed lookups
my $imdb= '/usr/share/mythtv/mythvideo/scripts/Movie/tmdb.py';
my $posters; 		#coverart directory
my $artdir; 		#fanart directory
my $new;		#-N flag
my $all;		#-A flag
my $sqlstat;		#used to store sql statments
my $stitle;		#grabbed from input for title search
my $dup;		#set if dupskip is on
my $manual;		#set if manual is on
my $host = hostname;	#the executing hosts machine name
my $fileup;		#set if fileup is on
my $exclude;		#set if exclude is on
my $currentimdb;	#dbs current imdb number before lookup is set
my $help;  		#pass to usage if help flag i set for extended
my $folder; 		#set if -Folder is set to overide DB for Videopath
my $title;		#Global Title to carry 
my $getcovers;		#-Getcovers flag
my $getfanart;		#-Getfanart flag
my $update;		#-Update flag
my $exrecheck;		#-Exrecheck flag
my $setimdbnum;		#User override for imdbnum
my $default;		#default override so all default entry on dups is assumed
my $newlevel;		#-Newlevel flag
my $showlevel;		#override for default showlevel of  for a folder or title



GetOptions("Posters=s"=>\$posters,
           "Artdir=s"=>\$artdir,
	   "A"=>\$all,
	   "N"=>\$new,
	   "Update"=>\$update,
	   "Exrecheck"=>\$exrecheck,
	   "Fileup"=>\$fileup,
	   "help"=>\$help,
	   "Exclude"=>\$exclude,
	   "Default"=>\$default,
	   "Manual"=>\$manual,
	   "Getcovers"=>\$getcovers,
	   "Getfanart"=>\$getfanart,
	   "Dupskip"=>\$dup,
	   "Title=s"=>\$stitle,
	   "Folder=s"=>\$folder,
	   "Newlevel=s"=>\$newlevel,
           "Host=s"=>\$dbhost,
           "Imdbnum=s"=>\$setimdbnum,
           "User=s"=>\$dbuser,
           "Imdb=s"=>\$imdb,
           "Password=s"=>\$dbpass);


if ( $help eq '1' ) {

	#Show extended usage
	usage($help);

}

#One quick sanity check here for anyone looking to -A and -Exclude together
if ( $exclude eq '1' && $all eq '1' ) {

print <<STOP;

**********************************Warning***************************************

	-Exclude and -A are not recommended to be used together. If they are 
        used together you may find yourself seriously pissed off. 

	-Exclude and -A will overwrite all information currently in the table.

        This means you will lose any manually entered IMDB numbers & data if 
        the script finds a match of any kind. 

	It is in your best interest to use -Exclude only with -N

	You have been warned. 

**********************************Warning***************************************

Hit [Enter] to exit, or type 'go' to continue with -Exclude and -A:
STOP

chomp(my $choice=<STDIN>);

	if ( "$choice" ne "go" ) {

		exit;

	}

}


#if the file update flag is set we'll update the db videometadata before continuing
if ( $fileup eq '1' ) {

        findnewvids($folder);

}

#Now parse sql options in an order that we care about
if ( $stitle ne "" ) {

	$sqlstat = "select inetref, filename, intid, title from videometadata where title like \"%$stitle%\" or filename like \"%$stitle%\"";

}

#Allow searching only on those videos in a particular directory
elsif ( $folder ne "" && $new eq '1' ) {
	
	$sqlstat = "select inetref, filename, intid, title from videometadata where (inetref=\"00000000\" or inetref=\"\") and filename like \"$folder%\" order by title";

}

elsif ( $folder ne "" && $all eq '1') {

	$sqlstat = "select inetref, filename, intid, title from videometadata where filename like \"$folder%\" order by title";

}

elsif ( $folder ne "" && $exrecheck eq '1') {

	$sqlstat = "select inetref, filename, intid, title from videometadata where inetref=\"99999999\" and filename like \"$folder%\" order by title";

}


elsif ( $all eq '1') {

	$sqlstat = "select inetref, filename, intId, title from videometadata order by title";

}

elsif ( $new eq '1' ) {

	$sqlstat = "select inetref, filename, intId, title from videometadata where inetref=\"00000000\" or inetref=\"\" order by title";

}

elsif ( $exrecheck eq '1' ) {

	$sqlstat = "select inetref, filename, intId, title from videometadata where inetref=\"99999999\" order by title";

}

elsif ( $getcovers eq '1' ) {

	$sqlstat = "select inetref,title from videometadata where coverfile like \"No Cover\" and inetref!=\"99999999\" and inetref!=\"00000000\"";
	
}

elsif ( $getfanart eq '1' ) {

	$sqlstat = "select inetref,title from videometadata where \(fanart is null or fanart like \"No Cover\"\) and inetref!=\"99999999\" and inetref!=\"00000000\"";

}

else {

	usage();

}	

#print "$sqlstat";

#Check to see if posters have been defined if not grab from db
if ( $posters eq "" ) {

	$posters=getposter();

}

#Retrieve the artwork directory if it's not set
if ( $artdir eq "" ) {

	$artdir=getartdir();

}

#Check for the posters directory & perms
if ( ! -d "$posters" || ! -w "$posters" ) {
	
	die "\nError: Posters path: $posters doesn't exist or is not writable\n\n";

}

#Check for imdb.pl script or the one defined
if ( ! -x "$imdb" ) {

	die "Error: $imdb is not executable\n";

}



if ( $getcovers eq '1' || $getfanart eq '1' ) {

	#Prepare the sql connection
	my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

	my $sth = $dbh->prepare($sqlstat);

	$sth->execute || die  "SQL statement failed";

	sep();
	while (my @row=$sth->fetchrow_array) {

		#Grab imdb number
		chomp($currentimdb = "$row[0]");

		#Get the title of the movie
		chomp($movie = "$row[1]");

		#Now required to query imdb data fully to get poster url
		undef my @TEMPDATA;
		@TEMPDATA=`$imdb -D $currentimdb`;

	        undef my $coverfile;
		undef my $coverurl;
		undef my $fanarturl;

		foreach (@TEMPDATA) {

			if ( $_ =~ /Coverart:/ ) {

				chomp((my $coverurl1)=(split /Coverart:/, $_)[1]);
				chomp(($coverurl)=(split /,/, $coverurl1)[0]);

			}

			if ( $_ =~ /Fanart:/ ) {

				chomp((my $fanarturl1)=(split /Fanart:/, $_)[1]);
				chomp(($fanarturl)=(split /,/, $fanarturl1)[0]);

			}

		}

		print "Searching for artwork for $movie\n";	

		if ( $getfanart eq "1" ) {

			#Grab the fanart
			$coverfile=covergrab("$currentimdb","$fanarturl","fanart");

		}
		else {

			#Grab the coverfile for the imdb	
			$coverfile=covergrab("$currentimdb","$coverurl","poster");
	
		}

		if ( $coverfile ne "No Cover" || $coverfile ne "NULL" ) {

			undef my $sqlstat;

	        	print "Updating videometadata with new artwork information\n";

			if ( $getfanart eq "1" ) {

                		$sqlstat="update videometadata set fanart=\"$coverfile\" where inetref=\"$currentimdb\"";

			}
			else {

                		$sqlstat="update videometadata set coverfile=\"$coverfile\" where inetref=\"$currentimdb\"";

			}

                	my $sth = $dbh->prepare($sqlstat);

                	$sth->execute || die  "SQL statement failed\n";

		}
		else {

			print "No artwork found for $movie\n";

		}
		sep();
	}

exit;
}	
		
#Grab allowable types because we'll use them for splitting the file name		
my $types=getypes();

#Prepare the sql connection 
my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

my $sth = $dbh->prepare($sqlstat);

$sth->execute || die  "SQL statement failed";

while (my @row=$sth->fetchrow_array) {

	#Grab current imdb number in DB
	chomp($currentimdb = "$row[0]");

	#Grab incremental id in DB for rewrite
	chomp($intid = "$row[2]");

        $movie = lc(basename($row[1]));
	#print "$movie\n";

	#Grab file id in DB for search
	chomp(($movie)=(split /$types/i, $movie)[0]);

	#print "$movie\n";

	#Grab title data 
	chomp(my $title= "$row[3]");

	sep();

	#Get's set if initial query returns duplicate data
	my $dupsfound;

	#print "current:$currentimdb , intid:$intid, movie: $movie, title: $title\n";

	#check to see if -Manual flag is set
	if ( $manual ne '1' && $update ne '1' && $setimdbnum eq '') {

		print "Searching for IMDB movieID for $movie\n";
		@TITLES=`$imdb -M "$movie"`;
		#print "$imdbnum";	
		#print "@TITLES";
	
		if ("$#TITLES" > '0') {

			$dupsfound="1";

		}
		else {

			chomp(($imdbnum)=(split /:/, $TITLES[0])[0]);

		}
	}	
	elsif ( $update eq '1' ) {

		#We won't search imdb.com on excludes or unknowns with this flag set
		if ( $currentimdb eq '00000000' || $currentimdb eq '99999999' ) {

			print "IMDB number for $movie is exluded(99999999) or unknown(00000000)\n";
			print "Maybe you want -Exrecheck or just a -N check instead\n";
			$imdbnum="";

		}

		else {

			print "Searching for IMDB data for $movie\n";	
			$imdbnum=$currentimdb;

		}

	}	
	elsif ( $setimdbnum ne '' ) {

		$imdbnum=$setimdbnum;
		chomp($imdbnum);
		print "Retrieving metadata for $movie with imdbnumber $imdbnum\n";

	}
	else {

		print "Enter the IMDB movieID for $movie\n";
                print "Enter the corresponding imdb number([Enter] to skip):";
		chomp($imdbnum=<STDIN>);
	}

	#We'll check the imdb ids here. If they're longer than expected we assume
	#it to be more than one return
	if ( $dupsfound eq '1') {

		#Check the multiple entries and skip if flag is set
		if ($dup ne '1' && $default ne '1') {

			#my @TITLES=`$imdb -M "$movie"`; 

			print "More than one match for $movie exists at IMDB\n";
	
			for (my $i=0;$i<=$#TITLES;$i++) {

				print "$i) $TITLES[$i]";

			}

                        print "Select a title ([0] Default, [x] to skip, [m] manual imdb number):";
			chomp(my $choice=<STDIN>);

			#print "my choice $choice\n";
			if ($choice eq "") {

				$choice="0";
				chomp($imdbnum=(split /:/, $TITLES[$choice]) [0]);
			}
				
			elsif ($choice eq "x") {

				$imdbnum="";
	                	push(@FAILEDLOOKUP, "$movie");


			}	

			elsif ($choice =~ /[0-9]/) {

				chomp($imdbnum=(split /:/, $TITLES[$choice]) [0]);
				#print $imdbnum;

			}
			elsif ($choice eq "m") {

				print "Enter IMDB number: ";
				chomp($imdbnum=<STDIN>);
			}

			else { 

				die "Not a valid selection\n";

			}
		}
		#If the default flag is set we'll assume the first entry in dups to be correct
		elsif ($default eq '1') {

			chomp($imdbnum=(split /:/, $TITLES[0]) [0]);

		}
		else {

			$imdbnum="";
			push(@FAILEDLOOKUP, "$movie");
		}

	}

	#Else check for failed lookups and -Exclude flag
	elsif ( "$imdbnum" eq "" && "$exclude" eq "1" && "$currentimdb" eq "00000000" ) {
		
		#We set a new bogus one to be ignored in later -N searches
		$imdbnum="99999999";

		push(@FAILEDLOOKUP, "$movie");

	}
		
	
	#Log the failed lookups here for later
	elsif ( "$imdbnum" eq "" ) {

		push(@FAILEDLOOKUP, "$movie");

		#print $#FAILEDLOOKUP;

	}
		
		
	
	#now check for imdbnumber from query
	if ( "$imdbnum" ne "" ) {
	
		undef my $coverfile;
		undef my $year;
		undef my $director;
		undef my $plot;
		undef my $userrating;
		undef my $rating;
		undef my $time;
		undef my $cast;
		undef my @CAST;
		undef my $genre;
		undef my @GENRE;
		undef my $country;
		undef my @COUNTRY;
		undef my $coverurl;
                undef my $url;
		undef my $fanarturl;
		undef my $fanartfile;

		#If bogus is set do this first as long as exclude recheck 0
		if ( "$imdbnum" eq "99999999" && "$exrecheck" ne "1" ) {

			#$title="$movie";	
			$coverfile="No Cover";
			$year="1895";
			$director="Unknown";
			$plot="Unknown";
			$userrating="0";
			$rating="NR";
			$time="0";

		}

		#Else go and fill in all necessary data
		else {

			#Now get the rest of the imdb data
			undef my @TEMPDATA;
			@TEMPDATA=`$imdb -D $imdbnum`;

			#Latest Version of imdb.pl caused my code to go out of wack based on the 
			#values being returned and the ordering. The following addtions are to 
			#try to make this more dynamic.
			#
			foreach (@TEMPDATA) {

				if ( $_ =~ /Title:\b/ ) {

					chomp(($title)=(split /Title:/, $_ )[1]);
					$title=~s/"//g;

				}
				if ( $_ =~ /Year:\b/ ) {

					chomp(($year)=(split /Year:/, $_)[1]);

				}

				if ( $_ =~ /Director:/ ) {

					chomp(($director)=(split /Director:/, $_)[1]);

				}

				if ( $_ =~ /Plot:/ ) {

					chomp(($plot)=(split /Plot:/, $_)[1]);
					$plot=~s/"//g;
					$plot=~s/'//g;

				}

				if ( $_ =~ /URL:/ ) {

					chomp(($url)=(split /URL:/, $_)[1]);

				}
				
				if ( $_ =~ /Fanart:/ ) {

					chomp((my $fanart1)=(split /Fanart: (.*)/, $_)[1]);
					chomp(($fanarturl)=(split /,/, $fanart1)[0]);

				}
				if ( $_ =~ /UserRating:/ ) {

					chomp(($userrating)=(split /UserRating:/, $_)[1]);

				}

				if ( $_ =~ /MovieRating:/ ) {

					chomp(($rating)=(split /MovieRating:/, $_)[1]);

				}

				if ( $_ =~ /Runtime:/ ) {

					chomp(($time)=(split /Runtime:/, $_)[1]);

				}
				if ( $_ =~ /Coverart:/ ) {

					chomp((my $coverurl1)=(split /Coverart:/, $_)[1]);
					chomp(($coverurl)=(split /,/, $coverurl1)[0]);

				}	
				if ( $_ =~ /Cast:/ ) {

					chomp(($cast)=(split /Cast:/, $_)[1]);
					$cast=~s/"//g;

					#If their is multiple entries split them
					if ($cast =~ /,/) {

						@CAST=(split /,/, $cast);

					}
					#If not multiple entries just push value into array
					else {

						push(@CAST,"$cast");

					}
				}
				if ( $_ =~ /Genres:/ ) {

					chomp(($genre)=(split /Genres:/, $_)[1]);
					
					#If their is multiple entries split them
					if ($genre=~ /,/) {

						@GENRE=(split /,/, $genre);

					}
					#If not multiple entries just push value into array
					else {

						push(@GENRE,"$genre");

					}

				}
				if ( $_ =~ /Countries:/ ) {

					chomp(($country)=(split /Countries:/, $_)[1]);
					
					#If their is multiple entries split them
					if ($country=~ /,/) {

						@COUNTRY=(split /,/, $country);

					}
					#If not multiple entries just push value into array
					else {

						push(@COUNTRY,"$country");

					}

				}

			}

			#New handler for coverart in 0.23 needed
			$coverfile=covergrab("$imdbnum","$coverurl","poster");

			#Retrieve the fanart now too
			$fanartfile=covergrab("$imdbnum","$fanarturl","fanart");

			#print $#TEMPDATA;
			#print "$title\n $year\n $director\n $plot\n $userrating\n $rating\n $time\n";
			#print "@CAST[0]";
			#print "@GENRE[0]";
			#print "@COUNTRY[0]";
			#exit;
		}

		#Check to see if the new showlevel is set
		if ( $newlevel !~ /[1-4]/ || not defined $newlevel) {

			$showlevel="1";

		}
		else {

			$showlevel="$newlevel";

		}

		#Last ditch check here cause sometimes imdb.pl comes back with empty values	
		#and updating the database with empty titles and data is really really irritating
		if ( $title ne "" ) {

			#Now let's write that data into the db
			print "Updating videometadata with new information\n";
			my $sqlstat="update videometadata set title=\"$title\", year=\"$year\", director=\"$director\", plot=\"$plot\", userrating=\"$userrating\", homepage=\"$url\", fanart=\"$fanartfile\", coverfile=\"$coverfile\", rating=\"$rating\", length=\"$time\", showlevel=\"$showlevel\", inetref=\"$imdbnum\" where intid=\"$intid\"";	


			my $sth = $dbh->prepare($sqlstat);

			$sth->execute || die  "SQL statement failed\n";


			#Let's process genre & cast information into their respective tables
			if (@GENRE ne "") {

				#First remove any prior entries for this movieid. (otherwise on -Update we'll make dups)
				my $sqlstat="delete from videometadatagenre where idvideo=\"$intid\"";
				my $sth = $dbh->prepare($sqlstat);
				$sth->execute || die  "SQL statement failed\n";

				print "Updating videometadatagenre with new information\n";
				
				foreach(@GENRE) {

					my $sqlstat="select intid from videogenre where genre=\"$_\""; 

					my $sth = $dbh->prepare($sqlstat);

					$sth->execute || die  "SQL statement failed";

					chomp(my $genreid=$sth->fetchrow_array);
					
					#If there is no match with genre search we create a new one		
					if (!$genreid) {
						
						#Add the genre if no id was previously returned
						my $sqlstat2="insert into videogenre set genre=\"$_\"";
						my $sth = $dbh->prepare($sqlstat2);
						$sth->execute || die  "SQL statement failed";

						#Now retrieve the id from the newly created one
						my $sth = $dbh->prepare($sqlstat);
						$sth->execute || die  "SQL statement failed";
						chomp($genreid=$sth->fetchrow_array);
					}	

					#Now add the genre pairing to videometagenre table
					my $sqlstat="insert into videometadatagenre set idvideo=\"$intid\", idgenre=\"$genreid\"";
					my $sth = $dbh->prepare($sqlstat);
					$sth->execute || die "SQL statement failed";

				}

			} 

			if (@CAST ne "") {

				#First remove any prior entries for this movieid. (otherwise on -Update we'll make dups)	
				my $sqlstat="delete from videometadatacast where idvideo=\"$intid\"";
                                my $sth = $dbh->prepare($sqlstat);
                                $sth->execute || die  "SQL statement failed\n";

                                print "Updating videometadatacast with new information\n";

				foreach(@CAST) {

					my $sqlstat="select intid from videocast where cast=\"$_\"";

                                        my $sth = $dbh->prepare($sqlstat);

                                        $sth->execute || die  "SQL statement failed";

                                        chomp(my $castid=$sth->fetchrow_array);

					#If there is no match with cast search then we'll add the actor to the database
					if (!$castid) {

						#Add the cast if no id was returned
						my $sqlstat2="insert into videocast set cast=\"$_\"";
 						my $sth = $dbh->prepare($sqlstat2);
                                                $sth->execute || die  "SQL statement failed";

                                                #Now retrieve the id from the newly created one
                                                my $sth = $dbh->prepare($sqlstat);
                                                $sth->execute || die  "SQL statement failed";
                                                chomp($castid=$sth->fetchrow_array);
					}

					#Now add the genre pairing to videometagenre table
                                        my $sqlstat="insert into videometadatacast set idvideo=\"$intid\", idcast=\"$castid\"";
                                        my $sth = $dbh->prepare($sqlstat);
                                        $sth->execute || die "SQL statement failed";
				
				}

			}

			if (@COUNTRY ne "") {

				#First remove any prior entries for this movieid. (otherwise on -Update we'll make dups)
				my $sqlstat="delete from videometadatacountry where idvideo=\"$intid\"";
				my $sth = $dbh->prepare($sqlstat);
				$sth->execute || die  "SQL statement failed\n";

				print "Updating videometadatacountry with new information\n";
				
				foreach(@COUNTRY) {

					my $sqlstat="select intid from videocountry where country=\"$_\""; 

					my $sth = $dbh->prepare($sqlstat);

					$sth->execute || die  "SQL statement failed";

					chomp(my $countryid=$sth->fetchrow_array);
					
					#If there is no match with country search we create a new one		
					if (!$countryid) {
						
						#Add the country if no id was previously returned
						my $sqlstat2="insert into videocountry set country=\"$_\"";
						my $sth = $dbh->prepare($sqlstat2);
						$sth->execute || die  "SQL statement failed";

						#Now retrieve the id from the newly created one
						my $sth = $dbh->prepare($sqlstat);
						$sth->execute || die  "SQL statement failed";
						chomp($countryid=$sth->fetchrow_array);
					}	

					#Now add the country pairing to videometadatacountry table
					my $sqlstat="insert into videometadatacountry set idvideo=\"$intid\", idcountry=\"$countryid\"";
					my $sth = $dbh->prepare($sqlstat);
					$sth->execute || die "SQL statement failed";

				}

			} 
					                                        
		}	

		else  {

			print "IMDB retrieval may have failed. Cannot update videometadata at this time\n";

		}


	}	
	#Update the newlevel if we've gotten here as well. Only good for non matches, to at least get level set
	elsif (defined $newlevel && $newlevel =~ /[1-4]/ ) {

                        #Now let's write that data into the db
                        print "Updating videometadata with new showlevel information\n";
                        my $sqlstat="update videometadata set showlevel=\"$newlevel\" where intid=\"$intid\"";

      			my $sth = $dbh->prepare($sqlstat);
			$sth->execute || die "SQL statement failed";

	}
	
}

if ( $#FAILEDLOOKUP >= 0 ) {

print "\n\nThe following videos had imdb lookup errors:\n @FAILEDLOOKUP\n";

}


sub usage {

	my $help=$_[0];
	my $comm=basename($0);

print <<STOP;

usage: $comm -[Required] -[Optional] -[Host] parameters

   Required parameters (Choose one):
     -A Get IMDB data for ALL movies in database
     -N Get IMDB data for movies with no IMDBID in database
     -Exrecheck Recheck videos previously excluded with -Exclude
     -Title Get IMDB data for movie(s) matching a quoted pattern

   Standalone options:
     -Getcovers Retrieve posters for any title that is missing one.
     -Getfanart Retrieve fanart for any title that is missing one.

   Optional parameters (Use in conjunction with options above):
     -Exclude Mark entries with no result with imdb placeholder (DEFAULT: OFF)
     -Fileup Search for new/removed videos and update database (DEFAULT: OFF)
     -Manual Query for manual IMDB number for each title (DEFAULT: OFF)
     -Dupskip Ignore the videos that find multiple imdb results (DEFAULT: OFF)
     -Default Select the first entry in duplicate match automatically (DEFAULT: OFF)
     -Update Use in conjuction with -A/Title to update videometadata (DEFAULT: OFF)
     -Folder Use in conjuction with -A/N to search specific directory
     -Imdbnum Pass in an imdbnum to use for all matches (see extended usage)
     -Newlevel Set the showlevel for a video or folder [1-4 valid]

    Host parameters:
     -Host Specify a database host (DEFAULT:$dbhost)
     -User Specify a database user (DEFAULT:$dbuser)
     -Password Specify a database password (DEFAULT:$dbpass)
     -Imdb Specify path to a working imdb.pl (DEFAULT:$imdb)
     -Posters Specify a cover art directory if different from entry in DB
     -Artdir Specify a fan art directory if different from entry in DB
   
   Example:
     $comm -N -Host myhostname -User mythuser -Password mythpass 

   Use $comm -help for extended examples
STOP

if ( $help eq '1' ) {
print <<STOP;

   New file check with IMDB lookup and non-local DB:
     $comm -N -Fileup -Host 192.168.1.50
  
   New file check with IMDB lookup and future exclusion for non-match:
     $comm -N -Fileup -Exclude 

   New file check, IMDB lookup for ALL movies, skipping multiple IMDB returns:
     $comm -A -Fileup -Dupskip
   
   Title search with Manual IMDB entry:
     $comm -Title "Austin Powers" -Manual

   Attempt to retrieve posters for any title missing one:
     $comm -Getcovers

   Retrieve metadata for all videos using a known IMDB number:
     $comm -A -Update

   Recheck for titles previously excluded:
     $comm -Exrecheck

   Use a specific IMDB number for all contents in a particular directory:
     $comm -A -Folder /My/movie/directory/band_of_brothers -Imdbnum 8888888
 
   Override the default showlevel for all content in a directory:
     $comm -A -Folder /My/movie/directory/Adult -Newlevel 4

STOP
}
exit;
}


#Get Posters directory from host if possible out of settings
sub getposter {

	my $sqlstat="select data from settings where hostname=\'$host\' and value=\'VideoArtworkDir\'";

	my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass") || die  usage();

	my $sth = $dbh->prepare($sqlstat); 

	$sth->execute || die  "SQL statement failed";

	chomp(my $posters=$sth->fetchrow_array);

	return($posters);

}

sub getartdir {

        my $sqlstat="select data from settings where hostname=\'$host\' and value=\'mythvideo.fanartDir\'";

        my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass") || die  usage();

        my $sth = $dbh->prepare($sqlstat);

        $sth->execute || die  "SQL statement failed";

        chomp(my $artdir=$sth->fetchrow_array);

        return($artdir);

}

#Routine to locate new & removed videos and update videometadata accordingly
sub findnewvids {

	my $folder=$_[0]; 
	my $videopath;

        sep();


	if ( $folder eq "" ) {

        print "Retrieving Video Path for this $host from $db.settings\n";

        #First get video directory path from settings database for exectuting host
        my $sqlstat="select data from settings where hostname=\'$host\' and value=\'VideoStartupDir\'";

        my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

        my $sth = $dbh->prepare($sqlstat);

        $sth->execute || die  "SQL statement failed";

        chomp($videopath=$sth->fetchrow_array);

	}
	else {

		print "Setting Video Path with manual override to $folder\n";

		chomp($videopath="$folder");

	}
        sep(1);

        #Now let's get the ignored file types so that we can truly ignore any overrides
        print "Retrieving ignored files extensions from $db.videometadata\n";

        $sqlstat = "select extension from videotypes where f_ignore=\"1\" order by extension";

        my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

        my $sth = $dbh->prepare($sqlstat);

        $sth->execute || die  "SQL statement failed";

        my @ITYPES;
        #put the ignored extensions found in the DB into an array
        while (my @row=$sth->fetchrow_array) {

                        push(@ITYPES,"@row");

        }

        sep(1);
        #Now look at allowed extensions and create an array to use when finding files
        #We'll append these to a list of pre-defined ones below

        print "Retrieving allowable file extensions from $db.videometadata\n";

        $sqlstat = "select extension from videotypes where f_ignore=\"0\" order by extension";

        my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

        my $sth = $dbh->prepare($sqlstat);

        $sth->execute || die  "SQL statement failed";


        #Array of commonly allowed video extensions
        my @TYPES=(".wmv",".avi",".vob",".iso",".bin",".mpeg",".mpg",".mp4",".mov",".mkv");

        #put the extensions found in the DB into an array
        while (my @row=$sth->fetchrow_array) {

                if ( "@TYPES" !~ /@row/ ) {

                        push(@TYPES,".@row");

                }


        }

        my @FTYPES;

        #Pull any ignored extensions out of final file type search string
        foreach (@TYPES) {

                if ( "@ITYPES" !~ /$_/ ) {

                        push(@FTYPES,"($_)\\Z|");

                }


        }

        my $types=join"",@FTYPES;

        #Take the last | off of the search string
        chop($types);

        sep(1);

	#Fix here to allow multiple folders specified in database
	my @VIDEOPATHS;
	if ( $videopath =~ /:/ )  {

		foreach(split /:/, $videopath) {	

			push(@VIDEOPATHS,"$_");

		}

	}
	else {

		push(@VIDEOPATHS,"$videopath");

	}

        #Search for all files that match our allow file extension patterns as defined in $types
        my @FILES;

    foreach (@VIDEOPATHS) {
        find ({wanted => sub{

                my $file=$File::Find::name ;
                if ( $file =~ /$types/i) {

			#Check to see if the file is a dvd
			if ( $file =~ /.vob/i || $file =~ /video_ts/i ) {

				chomp(my $dvddir=dirname($file));	

				#Now check to see if we're in a video_TS dir
				if ( $dvddir =~ /video_ts/i ) {

					#dirname again to get title
					chomp(my $dvdtitle=dirname($dvddir));
					
					$file="$dvdtitle";

				}
				else {

					#Assume title is parent directory
					$file="$dvddir";

				}
					
				#Now we need to make a quick check on last array value
				#to make sure we're not entering same title over and over
				if ($FILES[$#FILES] ne "$file" ) {

					push(@FILES,"$file");

				}	

			}		

			else {

	                        push(@FILES,"$file");

			}
                }

        },follow => 1}, "$_");

    }

        @FILES=sort(@FILES);

	my $totalfiles="$#FILES" + 1;

	if ($totalfiles ge '1') {

        	print "Found $totalfiles total videos in $videopath\n";
        	sep(1);

	}
	#else {

	#	die "No video files were found in $videopath\n";
	

	#}

        #Now lets see what's in videometadata
        print "Retrieving a list of known videos from $db.videometadata\n";

        sep(1);

    #Manage multiple paths
    my @LISTINDB;
    foreach (@VIDEOPATHS) {
        $sqlstat = "select filename from videometadata where filename like \"%$_%\" order by filename";

        my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

        my $sth = $dbh->prepare($sqlstat);

        $sth->execute || die  "SQL statement failed";

        #put those videos found in the DB into an array
        #my @LISTINDB;
        while (my @row=$sth->fetchrow_array) {

                push(@LISTINDB,"@row");

        }

    }

        print "Scrubbing the database for any videos that no longer exist in $videopath\n";

        my @FILESINDB;  #Database entries with an actual file associated
        #Now scrub database entries for any files that are no longer present
        foreach (@LISTINDB) {

		undef my $filename;	
		undef my $newfilename;

		#If the file doesn't exist we do a few checks before removing db entry
               	if (! -e "$_") {
			
			#Make sure location didn't change on filesystem
			chomp($filename=basename("$_"));
			#$title=~s/"//g;
			#$filename=~s/\[/\\[/g;
			for (my $file=0;$file<=$#FILES;$file++) {

				#my $vidfile=~s/[/\[/g;
				if ( $FILES[$file] =~ /$filename/ ) {

					$newfilename="$FILES[$file]";

				}

			}
		
			#Setup sqlstatment for file that may have changed location	
			if ( -e "$newfilename" ) {

				print "\nUpdating filename data for $_\n";

                     		$sqlstat = "update videometadata set filename=\"$newfilename\" where filename=\"$_\"";

				#Add to known files in database list
                       		push(@FILESINDB, "$newfilename");

                     		my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

                      		my $sth = $dbh->prepare($sqlstat);

                      		$sth->execute || die  "SQL statement failed";

			}
			#Remove from videmetadata and associated tables
			else {

				#Retrieve intid for the filename
				my $sqlstat = "select intid from videometadata where filename=\"$_\"";
				
				my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

	                        my $sth = $dbh->prepare($sqlstat);

                        	$sth->execute || die  "SQL statement failed";

				chomp(my $rmintid=$sth->fetchrow_array);

                     		my $sqlstat = "delete from videometadata where filename=\"$_\"";
				my $sqlstat2 = "delete from videometadatagenre where idvideo=\"$rmintid\"";
				my $sqlstat3 = "delete from videometadatacast where idvideo=\"$rmintid\"";
				my $sqlstat4 = "delete from videometadatacountry where idvideo=\"$rmintid\"";
                                my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

                      		print "\nRemoving entry for $_ from videometadata, videometadatacast, videometadatagenre, and videometadatacountry tables\n";
                                my $sth = $dbh->prepare($sqlstat);
                                $sth->execute || die  "SQL statement failed";

	                        my $sth = $dbh->prepare($sqlstat2);
	                        $sth->execute || die  "SQL statement failed";

                                my $sth = $dbh->prepare($sqlstat3);
                                $sth->execute || die  "SQL statement failed";

                                my $sth = $dbh->prepare($sqlstat4);
                                $sth->execute || die  "SQL statement failed";
			}

		}

               	else {

                    	#Add to new scrubbed list
                       	push(@FILESINDB, "$_");

                }


        }
        sep();


        my %count =();

        my @FILESDIFF;
        #Now well check to see what files differ between DB entries he Data
        foreach my $element (@FILESINDB, @FILES) {

                $count{$element}++;

        }
        foreach my $element (keys %count) {

                if ( $count{$element} < 2 ) {

                        #Add last check to see if the files are in db side
			#Need to handle () & [] properly in filename matches
			my $element1="$element";
			$element1=~s/\(/\\(/g;
			$element1=~s/\)/\\)/g;
			$element1=~s/\]/\\]/g;
			$element1=~s/\[/\\[/g;
                        if ( "@FILESINDB" !~ /$element1/) {

                        	push(@FILESDIFF, $element);
				print "$element\n";

			}
                }
        }

        #Now we tell the user how many updates to be made
	my $totaldiff="$#FILESDIFF" + 1;

	if ($totaldiff gt '0') {

        	print "Found a total of $totaldiff to be added to database\n";
        	sep (1);
	
	}
	else {

		print "Found no new videos to be added to database\n";

	}

        #Now we need to grab just a title and populate the db with
        foreach (@FILESDIFF) {

                my $title=basename($_,@TYPES);

                print "Adding entry for $_ to $db.videometadata\n";

                $sqlstat = "insert into videometadata (title,director,rating,inetref,coverfile,filename,showlevel) values(\"$title\",\"Unknown\",\"NR\",\"00000000\",\"No Cover\",\"$_\",\"1\");";

                my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

                my $sth = $dbh->prepare($sqlstat);

                $sth->execute || die  "SQL statement failed";

        }

        #print "@FILESDIFF\n";



}


#Is used to grab poster file & return coverfile
sub covergrab {

	my $imdbnum=$_[0];
	my $coverurl=$_[1];
	my $arttype=$_[2]; #Pass in poster or fanart
	undef my $posterdir;
	undef my $coverimage;	

	
	if ( "$arttype" eq "fanart" ) {

		$posterdir="$artdir";

	}
	else {
		
		$posterdir="$posters";

	}

        #Now lets find the cover image
        chomp($coverurl);


	if ($coverurl =~ /http:/) {

		if ($coverurl =~ /.png/) {
		
			$coverimage="$posterdir/$imdbnum.png";

		}	
		else {

			$coverimage="$posterdir/$imdbnum.jpg";

		}


                #Check to see if the poster is already there
		if ( ! -e "$coverimage" ) {

                	print "Retrieving $coverurl\n";
                       	my $jpegcontents=get("$coverurl") || print "Failed to retrieve $coverurl";
			if ( defined $jpegcontents ) {
                       		open(COVER,">$coverimage");
                       		print COVER $jpegcontents;
                       		close COVER;
			}
		}

	}
	else {

		$coverimage="No Cover";

	}

	return("$coverimage");

}


sub getypes {

	
        #Now let's get the ignored file types so that we can truly ignore any overrides
        print "Retrieving ignored files extensions from $db.videometadata\n";

        my $sqlstat = "select extension from videotypes where f_ignore=\"1\" order by extension";

        my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

        my $sth = $dbh->prepare($sqlstat);

        $sth->execute || die  "SQL statement failed";

        my @ITYPES;
        #put the ignored extensions found in the DB into an array
        while (my @row=$sth->fetchrow_array) {

                        push(@ITYPES,"@row");

        }

        sep(1);
        #Now look at allowed extensions and create an array to use when finding files
        #We'll append these to a list of pre-defined ones below

        print "Retrieving allowable file extensions from $db.videometadata\n";

        my $sqlstat = "select extension from videotypes where f_ignore=\"0\" order by extension";

        my $dbh = DBI->connect("dbi:mysql:$db:$dbhost","$dbuser","$dbpass");

        my $sth = $dbh->prepare($sqlstat);

        $sth->execute || die  "SQL statement failed";


        #Array of commonly allowed video extensions
        my @TYPES=(".wmv",".avi",".vob",".iso",".bin",".mpeg",".mpg",".mp4",".mov",".mkv");

        #put the extensions found in the DB into an array
        while (my @row=$sth->fetchrow_array) {

                if ( "@TYPES" !~ /@row/ ) {

                        push(@TYPES,".@row");

                }


        }

        my @FTYPES;

        #Pull any ignored extensions out of final file type search string
        foreach (@TYPES) {

                if ( "@ITYPES" !~ /$_/ ) {

                        push(@FTYPES,"($_)\\Z|");

                }


        }

        my $types=join"",@FTYPES;

        #Take the last | off of the search string
        chop($types);

	return "$types";

}


#Prints a easy on the eye separator
sub sep {

	if (defined $_[0]) {	

		print "=============================================================================\n";
		sleep $_[0];
	}
	else {

		print "=============================================================================\n";

	}

}
